import logging
import time
from enum import IntEnum
# noinspection PyUnresolvedReferences
from gpiozero import DigitalOutputDevice
from lightberrylib.device_protocols import IBaseDevice, IAlexaEnabledDevice


def get_utc_timestamp(seconds=None):
    return time.strftime("%Y-%m-%dT%H:%M:%S.00Z", time.gmtime(seconds))


class State(IntEnum):
    OFF = 0
    ON = 1


class Controller:
    def __init__(self, pin):
        self.__state__ = State.OFF
        self.__device__ = DigitalOutputDevice(pin)

    def set_status(self, state):
        if state == State.OFF:
            self.__device__.off()
        elif state == State.ON:
            self.__device__.on()
        self.__state__ = state
        return self.__state__

    def get_state(self):
        return self.__state__


class Device(IBaseDevice, IAlexaEnabledDevice):
    def __init__(self, device_id, pin, config):
        self.__device_id__ = device_id
        self.__config__ = config
        self.__controller__ = Controller(pin)

        print('Pairing code: {}'.format(self.__config__.get('pairingCode')))

    def get_id(self):
        return self.__device_id__

    def get_state(self):
        return {
            'data': 1 if self.__controller__.get_state() == State.ON else 0,
            'simple': 'On' if self.__controller__.get_state() == State.ON else 'Off',
            'alexa': [
                {
                    "namespace": "Alexa.PowerController",
                    "name": "powerState",
                    "value": 'ON' if self.__controller__.get_state() == State.ON else 'OFF',
                    "timeOfSample": get_utc_timestamp(),
                    "uncertaintyInMilliseconds": 500
                }
            ]
        }

    def set_state(self, state):
        if state == 0:
            self.__controller__.set_status(State.OFF)
        elif state == 1:
            self.__controller__.set_status(State.ON)

        logging.info('Setting state to {}'.format(self.get_state().get('simple')))

    def get_registration_data(self):
        return {
            'config': self.__config__,
            'state': self.get_state(),
        }

    def handle_alexa(self, request):
        logging.info('Alexa request received')
        logging.debug(request)

        if request['directive']['header']['namespace'] == 'Alexa.PowerController':
            if request['directive']['header']['name'] == 'TurnOn':
                self.set_state(1)
            elif request['directive']['header']['name'] == 'TurnOff':
                self.set_state(0)
